@@ -24,6 +24,8 @@
 //      Implementation for class ImportModule.
 //
 
+#include <sstream>
+
 //      utility stuff
 #include "macros.hh"
 #include "vector.hh"
@@ -66,10 +68,12 @@
 
 //	our stuff
 #include "ooSorts.cc"
+#include "pseudoParameters.cc"
 #include "renameModule.cc"
 #include "parameterization.cc"
 #include "instantiateModuleWithFreeParameters.cc"
 #include "instantiateModuleWithBoundParameters.cc"
+#include "latexImportModule.cc"
 
 ImportModule::ImportModule(int name, ModuleType moduleType)
   : MixfixModule(name, moduleType),
@@ -80,6 +84,7 @@ ImportModule::ImportModule(int name, ModuleType moduleType)
   //	object level syntax or meta-syntax, both of which are considered TEXT.
   //
   importPhase = UNVISITED;
+  parameterCopyParameterName = NONE;
   nrSortsFromParameters = 0;
   nrSymbolsFromParameters = 0;
   nrPolymorphsFromParameters = 0;
@@ -99,6 +104,7 @@ ImportModule::ImportModule(int name, ModuleType moduleType, Origin origin, Entit
   //
   addUser(parent);
   importPhase = UNVISITED;
+  parameterCopyParameterName = NONE;
   nrSortsFromParameters = 0;
   nrSymbolsFromParameters = 0;
   nrPolymorphsFromParameters = 0;
@@ -170,6 +176,25 @@ ImportModule::addImport(ImportModule* importedModule,
 		   QUOTE(this) << " not allowed.  Recovering by ignoring import.");
       return;
     }
+  if (!parameterNames.empty())
+    {
+      const EnclosingObject::ParameterSet& pseudos = importedModule->getPseudoParameters();
+      for (int id : parameterNames)
+	{
+	  if (pseudos.find(id) != pseudos.end())
+	    {
+	      IssueWarning(lineNumber << ": importation of " <<
+			   QUOTE(moduleTypeString(t)) << " " << QUOTE(importedModule) << " by " <<
+			   QUOTE(moduleTypeString(getModuleType())) << " " << QUOTE(this) <<
+			   " not allowed because the pseudo-parameter " << QUOTE(Token::name(id)) <<
+			   " of " << QUOTE(moduleTypeString(t)) << " " << QUOTE(importedModule) <<
+			   " would be captured by an actual parameter of " <<
+			   QUOTE(moduleTypeString(getModuleType())) << " " << QUOTE(this) <<
+			   ".");
+	      return;
+	    }
+	}
+    }
   importedModules.append(importedModule);
   importModes.append(mode);
   importedModule->addUser(this);
@@ -780,6 +805,8 @@ ImportModule::donateStatements2(ImportModule* importer, ImportTranslation& impor
 	    copy->setNonexec();
 	  if (e->isVariant())
 	    copy->setVariant();
+	  if (e->isExtension())
+	    copy->setExtension();
 	  copy->setLineNumber(e->getLineNumber());
 	  importer->insertEquation(copy);
 	  importer->checkEquation(copy);
@@ -806,6 +833,8 @@ ImportModule::donateStatements2(ImportModule* importer, ImportTranslation& impor
 	    copy->setNonexec();
 	  if (r->isNarrowing())
 	    copy->setNarrowing();
+	  if (r->isExtension())
+	    copy->setExtension();
 	  copy->setLineNumber(r->getLineNumber());
 	  importer->insertRule(copy);
 	  importer->checkRule(copy);
@@ -861,6 +890,82 @@ ImportModule::resetImportPhase()
     m->resetImportPhase();
 }
 
+void
+ImportModule::printModuleExpression(ostream& s, bool parameterBrackets) const
+{
+  //
+  //	We regenerate the module expression that was responsible for this module in a pretty way.
+  //
+  switch (origin)
+    {
+    case TEXT:
+      {
+	s << Token::name(id());
+	break;
+      }
+    case SUMMATION:
+      {
+	const char* sep = "";
+	for (const ImportModule* i : importedModules)
+	  {
+	    s << sep;
+	    sep = " + ";
+	    i->printModuleExpression(s, parameterBrackets);
+	  }
+	break;
+      }
+    case RENAMING:
+      {
+	if (baseModule->origin == SUMMATION)  // (M + N) * R
+	  s << '(';
+	baseModule->printModuleExpression(s, parameterBrackets);
+	if (baseModule->origin == SUMMATION)
+	  s << ')';
+	s << " * ";
+	canonicalRenaming->printRenaming(s, "(", ", ", true);
+	s << ')';
+	break;
+      }
+    case PARAMETER:
+      {
+	//
+	//	We need to print the name of a parameter copy of a theory like X :: T for show modules.
+	//
+	s << Token::name(parameterCopyParameterName) << " :: ";
+	baseModule->printModuleExpression(s, parameterBrackets);
+	break;
+      }
+    case INSTANTIATION:
+      {
+	if (baseModule->origin == RENAMING)  // (M * R){...}, can't be SUMMATION because of unbound parameters
+	  s << '(';
+	baseModule->printModuleExpression(s, parameterBrackets);
+	if (baseModule->origin == RENAMING)
+	  s << ')';
+	const char* sep = "{";
+	for (const Argument* a : savedArguments)
+	  {
+	    s << sep;
+	    sep = ", ";
+	    if (const View* v = dynamic_cast<const View*>(a))
+	      v->printViewExpression(s, parameterBrackets);  // FIXME
+	    else if (const Parameter* p = dynamic_cast<const Parameter*>(a))
+	      {
+		if (parameterBrackets)
+		  s <<  "[";
+		s << Token::name(p->id());
+		if (parameterBrackets)
+		  s <<  "]";
+	      }
+	  }
+	s << '}';
+	break;
+      }
+    default:
+      CantHappen("bad module origin");
+    }
+}
+
 #ifndef NO_ASSERT
 void
 ImportModule::dumpImports(ostream& s) const
